# =====================================
# generator=datazen
# version=3.1.3
# hash=dcfab6b503a06109b92ab9ee84f0bec4
# =====================================

"""
Useful defaults and other package metadata.
"""

DESCRIPTION = "An interface generator for distributed computing."
PKG_NAME = "ifgen"
VERSION = "1.2.0"
